/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file    app_entry.c
  * @author  MCD Application Team
  * @brief   Entry point of the application
  ******************************************************************************
  * @attention
  *
  * Copyright (c) 2022 STMicroelectronics.
  * All rights reserved.
  *
  * This software is licensed under terms that can be found in the LICENSE file
  * in the root directory of this software component.
  * If no LICENSE file comes with this software, it is provided AS-IS.
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "app_common.h"
#include "log_module.h"
#include "app_conf.h"
#include "main.h"
#include "app_entry.h"
#include "stm32_rtos.h"
#if (CFG_LPM_LEVEL != 0)
#include "stm32_lpm.h"
#endif /* (CFG_LPM_LEVEL != 0) */
#include "stm32_timer.h"
#include "advanced_memory_manager.h"
#include "stm32_mm.h"
#if (CFG_LOG_SUPPORTED != 0)
#include "stm32_adv_trace.h"
#include "serial_cmd_interpreter.h"
#endif /* CFG_LOG_SUPPORTED */
#include "app_ble.h"
#include "ll_sys.h"
#include "ll_sys_if.h"
#include "app_sys.h"
#include "otp.h"
#include "scm.h"
#include "bpka.h"
#include "flash_driver.h"
#include "flash_manager.h"
#include "simple_nvm_arbiter.h"
#include "app_debug.h"

/* Private includes -----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "codec_mngr.h"
#include "codec_if.h"
#include "stm32wba55g_discovery.h"
#include "stm32wba55g_discovery_audio.h"
#include "stm32wba55g_discovery_bus.h"
#if (CFG_LCD_SUPPORTED == 1)
#include "stm32wba55g_discovery_lcd.h"
#include "stm32_lcd.h"
#endif /* CFG_LCD_SUPPORTED */
#include "stm32_lpm.h"
#include "app_menu.h"
#include "app_menu_cfg.h"
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/

/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private defines -----------------------------------------------------------*/

/* USER CODE BEGIN PD */

/* USER CODE END PD */

/* Private macros ------------------------------------------------------------*/
/* USER CODE BEGIN PM */
#define VOLUME_VCP_TO_VOLUME_BSP(Volume) ((uint32_t) (47.4f*log10f(((float) Volume)/2+1)))
/* USER CODE END PM */

/* Private constants ---------------------------------------------------------*/
/* USER CODE BEGIN PC */

/* USER CODE END PC */

/* Private variables ---------------------------------------------------------*/
#if ( CFG_LPM_LEVEL != 0)
static bool system_startup_done = FALSE;
#endif /* ( CFG_LPM_LEVEL != 0) */

#if (CFG_LOG_SUPPORTED != 0)
/* Log configuration */
static Log_Module_t Log_Module_Config = { .verbose_level = APPLI_CONFIG_LOG_LEVEL, .region = LOG_REGION_ALL_REGIONS };
#endif /* (CFG_LOG_SUPPORTED != 0) */

/* AMM configuration */
static uint32_t AMM_Pool[CFG_AMM_POOL_SIZE];
static AMM_VirtualMemoryConfig_t vmConfig[CFG_AMM_VIRTUAL_MEMORY_NUMBER] =
{
  /* Virtual Memory #1 */
  {
    .Id = CFG_AMM_VIRTUAL_STACK_BLE,
    .BufferSize = CFG_AMM_VIRTUAL_STACK_BLE_BUFFER_SIZE
  },
  /* Virtual Memory #2 */
  {
    .Id = CFG_AMM_VIRTUAL_APP_BLE,
    .BufferSize = CFG_AMM_VIRTUAL_APP_BLE_BUFFER_SIZE
  },
};

static AMM_InitParameters_t ammInitConfig =
{
  .p_PoolAddr = AMM_Pool,
  .PoolSize = CFG_AMM_POOL_SIZE,
  .VirtualMemoryNumber = CFG_AMM_VIRTUAL_MEMORY_NUMBER,
  .p_VirtualMemoryConfigList = vmConfig
};

/* USER CODE BEGIN PV */
#if (CFG_JOYSTICK_SUPPORTED == 1)
static int32_t Joystick_Prev_State;
static UTIL_TIMER_Object_t JOYSTICK_TimerObj;
#endif /* (CFG_JOYSTICK_SUPPORTED == 1) */
#if (CFG_LCD_SUPPORTED == 1)
static uint32_t Codec_Frequency = 0;
static Audio_Role_t Audio_Role = 0;
#endif /* (CFG_LCD_SUPPORTED == 1) */
static uint32_t PLL_Target_Clock_Freq = 0;
static uint8_t MxAudioInit_Flag = 0;
uint32_t Sink_frame_size = 0;
uint32_t Source_frame_size = 0;
static uint8_t Record_Req_Pause = 0;
static uint8_t Play_Req_Pause = 0;
static uint32_t Current_Volume = 50;
/* USER CODE END PV */

/* Global variables ----------------------------------------------------------*/
/* USER CODE BEGIN GV */
#if (CFG_JOYSTICK_SUPPORTED == 1)
uint8_t JOY_StandbyExitFlag = 0;
#endif /* (CFG_JOYSTICK_SUPPORTED == 1) */
/* USER CODE END GV */

/* Private functions prototypes-----------------------------------------------*/
static void System_Init( void );
static void SystemPower_Config( void );
static void Config_HSE(void);
static void APPE_RNG_Init( void );

static void APPE_AMM_Init(void);
static void AMM_WrapperInit(uint32_t * const p_PoolAddr, const uint32_t PoolSize);
static uint32_t * AMM_WrapperAllocate(const uint32_t BufferSize);
static void AMM_WrapperFree(uint32_t * const p_BufferAddr);

static void APPE_FLASH_MANAGER_Init( void );

static void APPE_BPKA_Init( void );

/* USER CODE BEGIN PFP */
#if (CFG_LED_SUPPORTED == 1)
static void Led_Init(void);
#endif /* (CFG_LED_SUPPORTED == 1) */
#if (CFG_JOYSTICK_SUPPORTED == 1)
static void Joystick_Init( uint8_t wkup_mode );
static void Joystick_TimerCallback(void *arg);
static void Joystick_ActionHandle(void);
#endif /* CFG_JOYSTICK_SUPPORTED */
#if (CFG_LCD_SUPPORTED == 1)
static void LCD_Init(void);
#endif /* CFG_LCD_SUPPORTED */
static void Init_AudioBuffer(uint8_t *pSnkBuff, uint16_t SnkBuffLen, uint8_t *pSrcBuff, uint16_t SrcBuffLen);
static void AudioClock_Deinit(void);
static void PLL_Ready_Task(void);
static void BSP_MIC_Gain_Init(uint8_t mode);
/* USER CODE END PFP */

/* External variables --------------------------------------------------------*/

/* USER CODE BEGIN EV */

/* USER CODE END EV */

/* Functions Definition ------------------------------------------------------*/
/**
 * @brief   Wireless Private Area Network configuration.
 */
void MX_APPE_Config(void)
{
  /* Configure HSE Tuning */
  Config_HSE();
}

/**
 * @brief   Wireless Private Area Network initialisation.
 */
uint32_t MX_APPE_Init(void *p_param)
{
  APP_DEBUG_SIGNAL_SET(APP_APPE_INIT);

  UNUSED(p_param);

  /* System initialization */
  System_Init();

  /* Configure the system Power Mode */
  SystemPower_Config();

  /* Initialize the Advance Memory Manager module */
  APPE_AMM_Init();

  /* Initialize the Random Number Generator module */
  APPE_RNG_Init();

  /* Initialize the Flash Manager module */
  APPE_FLASH_MANAGER_Init();

  /* Disable flash before any use - RFTS */
  FD_SetStatus (FD_FLASHACCESS_RFTS, LL_FLASH_DISABLE);
  /* Enable RFTS Bypass for flash operation - Since LL has not started yet */
  FD_SetStatus (FD_FLASHACCESS_RFTS_BYPASS, LL_FLASH_ENABLE);
  /* Enable flash system flag */
  FD_SetStatus (FD_FLASHACCESS_SYSTEM, LL_FLASH_ENABLE);

  /* USER CODE BEGIN APPE_Init_1 */
#if (CFG_LED_SUPPORTED == 1)
  Led_Init();
#endif /* (CFG_LED_SUPPORTED == 1) */
#if (CFG_JOYSTICK_SUPPORTED == 1)
  Joystick_Init(0);
#endif /* (CFG_JOYSTICK_SUPPORTED == 1) */
#if (CFG_LCD_SUPPORTED == 1)
  LCD_Init();
  UTIL_SEQ_RegTask(1U << CFG_TASK_MENU_PRINT_ID, UTIL_SEQ_RFU, Menu_Print_Task);
#endif /* CFG_LCD_SUPPORTED */

  UTIL_SEQ_RegTask(1U << CFG_TASK_PLL_READY_ID, UTIL_SEQ_RFU, PLL_Ready_Task);
  /* USER CODE END APPE_Init_1 */

  /* Initialize the Ble Public Key Accelerator module */
  APPE_BPKA_Init();

  /* Initialize the Simple Non Volatile Memory Arbiter */
  if( SNVMA_Init((uint32_t *)CFG_SNVMA_START_ADDRESS) != SNVMA_ERROR_OK )
  {
    Error_Handler();
  }

  APP_BLE_Init();

  /* Disable RFTS Bypass for flash operation - Since LL has not started yet */
  FD_SetStatus (FD_FLASHACCESS_RFTS_BYPASS, LL_FLASH_DISABLE);

  /* USER CODE BEGIN APPE_Init_2 */
#if (CFG_JOYSTICK_SUPPORTED == 1)
  /* Register Button Tasks */
  UTIL_SEQ_RegTask(1U << CFG_TASK_JOYSTICK_ID, UTIL_SEQ_RFU, Joystick_ActionHandle);

  /* Create periodic timer for joystick position reading */
  UTIL_TIMER_Create(&JOYSTICK_TimerObj, 100, UTIL_TIMER_PERIODIC, &Joystick_TimerCallback, 0);
  UTIL_TIMER_Start(&JOYSTICK_TimerObj);
#endif /* CFG_JOYSTICK_SUPPORTED */

  /* Indicate to the low power manager that the Stop mode isn't allow : Sleep mode will be used in low power mode*/
  UTIL_LPM_SetStopMode(1U << CFG_LPM_APP, UTIL_LPM_DISABLE);

  /* USER CODE END APPE_Init_2 */

  APP_DEBUG_SIGNAL_RESET(APP_APPE_INIT);

  return WPAN_SUCCESS;
}

void MX_APPE_Process(void)
{
  /* USER CODE BEGIN MX_APPE_Process_1 */

  /* USER CODE END MX_APPE_Process_1 */
  UTIL_SEQ_Run(UTIL_SEQ_DEFAULT);
  /* USER CODE BEGIN MX_APPE_Process_2 */

  /* USER CODE END MX_APPE_Process_2 */
}

/* USER CODE BEGIN FD */

/* USER CODE END FD */

/*************************************************************
 *
 * LOCAL FUNCTIONS
 *
 *************************************************************/

/**
 * @brief Configure HSE by read this Tuning from OTP
 *
 */
static void Config_HSE(void)
{
  OTP_Data_s* otp_ptr = NULL;

  /* Read HSE_Tuning from OTP */
  if (OTP_Read(DEFAULT_OTP_IDX, &otp_ptr) != HAL_OK)
  {
    /* OTP no present in flash, apply default gain */
    HAL_RCCEx_HSESetTrimming(0x0C);
  }
  else
  {
    HAL_RCCEx_HSESetTrimming(otp_ptr->hsetune);
  }
}

/**
 *
 */
static void System_Init( void )
{
  /* Clear RCC RESET flag */
  LL_RCC_ClearResetFlags();

  UTIL_TIMER_Init();

  /* Enable wakeup out of standby from RTC ( UTIL_TIMER )*/
  HAL_PWR_EnableWakeUpPin(PWR_WAKEUP_PIN7_HIGH_3);

#if (CFG_LOG_SUPPORTED != 0)
  /* Initialize the logs ( using the USART ) */
  Log_Module_Init( Log_Module_Config );

  /* Initialize the Command Interpreter */
  Serial_CMD_Interpreter_Init();
#endif  /* (CFG_LOG_SUPPORTED != 0) */

#if(CFG_RT_DEBUG_DTB == 1)
  /* DTB initialization and configuration */
  RT_DEBUG_DTBInit();
  RT_DEBUG_DTBConfig();
#endif /* CFG_RT_DEBUG_DTB */

#if ( CFG_LPM_LEVEL != 0)
  system_startup_done = TRUE;
#endif /* ( CFG_LPM_LEVEL != 0) */

  return;
}

/**
 * @brief  Configure the system for power optimization
 *
 * @note  This API configures the system to be ready for low power mode
 *
 * @param  None
 * @retval None
 */
static void SystemPower_Config(void)
{
#if (CFG_SCM_SUPPORTED == 1)
  /* Initialize System Clock Manager */
  scm_init();
#endif /* CFG_SCM_SUPPORTED */

#if (CFG_DEBUGGER_LEVEL == 0)
  /* Pins used by SerialWire Debug are now analog input */
  GPIO_InitTypeDef DbgIOsInit = {0};
  DbgIOsInit.Mode = GPIO_MODE_ANALOG;
  DbgIOsInit.Pull = GPIO_NOPULL;
  DbgIOsInit.Pin = GPIO_PIN_13|GPIO_PIN_14|GPIO_PIN_15;
  __HAL_RCC_GPIOA_CLK_ENABLE();
  HAL_GPIO_Init(GPIOA, &DbgIOsInit);

  DbgIOsInit.Mode = GPIO_MODE_ANALOG;
  DbgIOsInit.Pull = GPIO_NOPULL;
  DbgIOsInit.Pin = GPIO_PIN_3|GPIO_PIN_4;
  __HAL_RCC_GPIOB_CLK_ENABLE();
  HAL_GPIO_Init(GPIOB, &DbgIOsInit);
#endif /* CFG_DEBUGGER_LEVEL */

#if (CFG_LPM_LEVEL != 0)
  /* Initialize low Power Manager. By default enabled */
  UTIL_LPM_Init();

#if (CFG_LPM_STDBY_SUPPORTED == 1)
  /* Enable SRAM1, SRAM2 and RADIO retention*/
  LL_PWR_SetSRAM1SBRetention(LL_PWR_SRAM1_SB_FULL_RETENTION);
  LL_PWR_SetSRAM2SBRetention(LL_PWR_SRAM2_SB_FULL_RETENTION);
  LL_PWR_SetRadioSBRetention(LL_PWR_RADIO_SB_FULL_RETENTION); /* Retain sleep timer configuration */

#else /* (CFG_LPM_STDBY_SUPPORTED == 1) */
  UTIL_LPM_SetOffMode(1U << CFG_LPM_APP, UTIL_LPM_DISABLE);
#endif /* (CFG_LPM_STDBY_SUPPORTED == 1) */
#endif /* (CFG_LPM_LEVEL != 0)  */

  /* USER CODE BEGIN SystemPower_Config */

  /* USER CODE END SystemPower_Config */
}

/**
 * @brief Initialize Random Number Generator module
 */
static void APPE_RNG_Init(void)
{
  HW_RNG_Start();

  /* Register Random Number Generator task */
  UTIL_SEQ_RegTask(1U << CFG_TASK_HW_RNG, UTIL_SEQ_RFU, (void (*)(void))HW_RNG_Process);
}

/**
 * @brief Initialize Flash Manager module
 */
static void APPE_FLASH_MANAGER_Init(void)
{
  /* Register Flash Manager task */
  UTIL_SEQ_RegTask(1U << CFG_TASK_FLASH_MANAGER, UTIL_SEQ_RFU, FM_BackgroundProcess);
}

/**
 * @brief Initialize Ble Public Key Accelerator module
 */
static void APPE_BPKA_Init(void)
{
  /* Register Ble Public Key Accelerator task */
  UTIL_SEQ_RegTask(1U << CFG_TASK_BPKA, UTIL_SEQ_RFU, BPKA_BG_Process);
}

static void APPE_AMM_Init(void)
{
  /* Initialize the Advance Memory Manager */
  if( AMM_Init(&ammInitConfig) != AMM_ERROR_OK )
  {
    Error_Handler();
  }

  /* Register Advance Memory Manager task */
  UTIL_SEQ_RegTask(1U << CFG_TASK_AMM, UTIL_SEQ_RFU, AMM_BackgroundProcess);
}

static void AMM_WrapperInit(uint32_t * const p_PoolAddr, const uint32_t PoolSize)
{
  UTIL_MM_Init ((uint8_t *)p_PoolAddr, ((size_t)PoolSize * sizeof(uint32_t)));
}

static uint32_t * AMM_WrapperAllocate(const uint32_t BufferSize)
{
  return (uint32_t *)UTIL_MM_GetBuffer (((size_t)BufferSize * sizeof(uint32_t)));
}

static void AMM_WrapperFree (uint32_t * const p_BufferAddr)
{
  UTIL_MM_ReleaseBuffer ((void *)p_BufferAddr);
}

/* USER CODE BEGIN FD_LOCAL_FUNCTIONS */
#if (CFG_LED_SUPPORTED == 1)
static void Led_Init( void )
{
  /* Leds Initialization */
  BSP_LED_Init(LD3);
  BSP_LED_On(LD3);

  return;
}
#endif /* CFG_LED_SUPPORTED */

#if (CFG_JOYSTICK_SUPPORTED == 1)
static void Joystick_Init( uint8_t wkup_mode )
{
  if (wkup_mode == 1)
  {
    /* configuration as a WKUP Pin */
    GPIO_InitTypeDef GPIO_InitStruct = {0};
    GPIO_InitStruct.Pin       = JOY1_CHANNEL_GPIO_PIN;
    GPIO_InitStruct.Mode      = GPIO_MODE_IT_FALLING;
    GPIO_InitStruct.Pull      = GPIO_NOPULL;
    GPIO_InitStruct.Speed     = GPIO_SPEED_FREQ_HIGH;
    HAL_GPIO_Init(JOY1_CHANNEL_GPIO_PORT, &GPIO_InitStruct);

    HAL_NVIC_SetPriority(EXTI1_IRQn, 15, 0);
    HAL_NVIC_EnableIRQ(EXTI1_IRQn);
  }
  else
  {
    BSP_JOY_Init(JOY1, JOY_MODE_POLLING, JOY_ALL);
    /* reconfiguration of the ADC4 interrupt priority */
    HAL_NVIC_DisableIRQ(ADC4_IRQn);
    HAL_NVIC_SetPriority(ADC4_IRQn, 15, 0);
    HAL_NVIC_EnableIRQ(ADC4_IRQn);
  }
}

static void Joystick_TimerCallback(void *arg)
{
  UTIL_SEQ_SetTask( 1U << CFG_TASK_JOYSTICK_ID, CFG_SEQ_PRIO_0);
}

static void Joystick_ActionHandle(void)
{
  /* Joystick reinitialization */
  Joystick_Init(0);

  int32_t state = BSP_JOY_GetState(JOY1);

  BSP_JOY_DeInit(JOY1, JOY_ALL);

  /* process Joystick information */
  if (state != JOY_NONE && state != Joystick_Prev_State)
  {
    if (state == JOY_SEL)
    {
    }
    else if (state == JOY_UP)
    {
      Menu_Up();
    }
    else if (state == JOY_RIGHT)
    {
      Menu_Right();
    }
    else if (state == JOY_DOWN)
    {
      Menu_Down();
    }
    else if (state == JOY_LEFT)
    {
      Menu_Left();
    }
  }

  Joystick_Prev_State = state;
}
#endif  /* CFG_JOYSTICK_SUPPORTED */

#if (CFG_LCD_SUPPORTED == 1)

static void LCD_Init( void )
{
  /* BSP init LCD*/
  BSP_LCD_Init(0, LCD_ORIENTATION_LANDSCAPE);

  /* Set LCD Foreground Layer  */
  UTIL_LCD_SetFuncDriver(&LCD_Driver); /* SetFunc before setting device */
  UTIL_LCD_SetDevice(0);               /* SetDevice after funcDriver is set */

  BSP_LCD_DisplayOn(0);

  BSP_LCD_Clear(0,SSD1315_COLOR_BLACK);
  BSP_LCD_Refresh(0);

  /* Set the LCD Text Color */
  UTIL_LCD_SetFont(&Font12);
  UTIL_LCD_SetTextColor(SSD1315_COLOR_WHITE);
  UTIL_LCD_SetBackColor(SSD1315_COLOR_BLACK);
  BSP_LCD_Refresh(0);

  /* release bus for power optimisation */
  BSP_SPI3_DeInit();
}

#endif  /* CFG_LCD_SUPPORTED */


/**
  * @brief Configure PLL for the audio frequency, PLL output is used for the SAI peripheral and as the core clock
  * @retval None
  */
void AudioClock_Init(uint32_t audio_frequency_type)
{
  uint32_t target_freq;
  Codec_Frequency = audio_frequency_type;
  if (audio_frequency_type == SAMPLE_FREQ_44100_HZ)
  {
    target_freq = 90316800;
  }
  else
  {
    /* valid for 8k, 16k, 24k, 32k or 48k */
    target_freq = 98304000;
  }

  if ( target_freq !=  PLL_Target_Clock_Freq )
  {
    scm_pll_config_t pll_config;
    LOG_INFO_APP("Audio Clock Initialization\n");

    pll_config.pll_mode = PLL_FRACTIONAL_MODE;
    pll_config.PLLM = 6;
    pll_config.PLLP = 4;
    pll_config.PLLQ = 4;
    pll_config.PLLR = 4;

    /* VCO frequency should be in range 128 to 544 MHz */
    /* PLL fractionnal should allows to cover +/-550ppm around the targeted frequency*/
    if (target_freq == 90316800)
    {
      pll_config.PLLN = 67;  /* VCO = HSE/M * N * (1 + Nfrac / 8192) < 32 / 6 * (67 + 1) = 362.7 MHz */
      pll_config.PLLFractional = 6042;
      pll_config.AHB5_PLL1_CLKDivider = LL_RCC_AHB5_DIV_3;
      /* PLLSYS = 90.31673 MHz */
    }
    else
    {
      pll_config.PLLN = 73;  /* VCO = HSE/M * N * (1 + Nfrac / 8192) < 32 / 6 * (73 + 1) = 394.7 MHz */
      pll_config.PLLFractional = 5964;
      pll_config.AHB5_PLL1_CLKDivider = LL_RCC_AHB5_DIV_4;
      /* PLLSYS = 98.30396 MHz */
    }

    UTIL_LPM_SetStopMode(1 << CFG_LPM_AUDIO, UTIL_LPM_DISABLE);

    scm_pll_setconfig(&pll_config);

    scm_setsystemclock(SCM_USER_APP, SYS_PLL);

    /* enable P output used for the audio timer */
    LL_RCC_PLL1_EnableDomain_PLL1P();

    PLL_Target_Clock_Freq = target_freq;
  }
  else
  {
    LOG_INFO_APP("Audio Clock already configured\n");
  }
}

void PLL_Ready_ProcessIT(void)
{
  UTIL_SEQ_SetTask(1U << CFG_TASK_PLL_READY_ID, CFG_SEQ_PRIO_0);
}

void PLL_Ready_Task(void)
{
  /* set Link Layer audio timings */
  Evnt_timing_t event_time;
  event_time.drift_time    = ISO_PLL_DRIFT_TIME;
  event_time.exec_time     = ISO_PLL_EXEC_TIME;
  event_time.schdling_time = ISO_PLL_SCHDL_TIME;
  ll_intf_config_schdling_time(&event_time);

  CODEC_CLK_Init();

  AUDIO_PLLConfig_t corrector_pll_config;
  corrector_pll_config.PLLTargetFreq = PLL_Target_Clock_Freq;
  corrector_pll_config.VCOInputFreq = (32000000.0f / 6.0f); /* HSE / PLL_M */
  corrector_pll_config.PLLOutputDiv = 4; /* PLL_P */
  AUDIO_InitializeClockCorrector(&corrector_pll_config, 500, 4000);

  /* I2C init takes a long time, it is better to do it before CIS establishement */
  BSP_I2C3_Init();
}

void PLL_Exit(void)
{
  if (PLL_Target_Clock_Freq != 0)
  {
    scm_setsystemclock(SCM_USER_APP, NO_CLOCK_CONFIG);

    PLL_Target_Clock_Freq = 0;

    UTIL_LPM_SetStopMode(1 << CFG_LPM_AUDIO, UTIL_LPM_ENABLE);
  }
}


static void AudioClock_Deinit( void )
{
  /* back to default timings */
  Evnt_timing_t event_time;
  event_time.drift_time    = DEFAULT_DRIFT_TIME;
  event_time.exec_time     = DEFAULT_EXEC_TIME;
  event_time.schdling_time = DEFAULT_SCHDL_TIME;
  ll_intf_config_schdling_time(&event_time);

  LOG_INFO_APP("Audio Clock Deinitialization\n");

  AUDIO_DeinitializeClockCorrector();
}

HAL_StatusTypeDef MX_SAI1_ClockConfig(SAI_HandleTypeDef *hsai, uint32_t SampleRate)
{
  /* PLL already configured */

  RCC_PeriphCLKInitTypeDef PeriphClkInit = {0};

  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_SAI1;
  PeriphClkInit.Sai1ClockSelection = RCC_SAI1CLKSOURCE_PLL1P;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    Error_Handler();
  }

  __HAL_RCC_SAI1_CLK_ENABLE();

  return HAL_OK;
}

/**
  * @brief Adjust microphone gain using 3 pre-set configuration
  * @note  To be used at the initialization only, after BSP_AUDIO_xx_Init()
  * @param mode : value from 0 to 2, 0 for lowest gain and 2 for the highest
  */
static void BSP_MIC_Gain_Init(const uint8_t mode){

  const uint8_t Tab_mic_ctrl[3] =   {0x80, 0x00, 0x03};
  const uint8_t Tab_attenuator[3] = {0xF1, 0xF6, 0x00};

  if( mode > 2)
  {
    Error_Handler();
  }

  uint8_t attenuator_gain = Tab_attenuator[mode];
  uint8_t mic_control = Tab_mic_ctrl[mode];

  BSP_I2C3_WriteReg(AUDIO_I2C_ADDRESS, CS42L51_ADCA_ATTENUATOR, &attenuator_gain, 0x01);
  BSP_I2C3_WriteReg(AUDIO_I2C_ADDRESS, CS42L51_MIC_CTRL, &mic_control, 0x01);
}

void MX_AudioInit(Audio_Role_t role,
                  Sampling_Freq_t sampling_frequency,
                  Frame_Duration_t frame_duration,
                  uint8_t *pSnkBuff,
                  uint8_t *pSrcBuff)
{
  uint32_t sample_per_frame;
  uint32_t audioFrequency;
  uint32_t channel_at_src;
  BSP_AUDIO_Init_t audio_conf;

  switch (sampling_frequency)
  {
    case SAMPLE_FREQ_8000_HZ:
      audioFrequency = SAI_AUDIO_FREQUENCY_8K;
      break;

    case SAMPLE_FREQ_16000_HZ:
      audioFrequency = SAI_AUDIO_FREQUENCY_16K;
      break;

    case SAMPLE_FREQ_22050_HZ:
      audioFrequency = SAI_AUDIO_FREQUENCY_22K;
      break;

    case SAMPLE_FREQ_24000_HZ:
      audioFrequency = SAI_AUDIO_FREQUENCY_24K;
      break;

    case SAMPLE_FREQ_32000_HZ:
      audioFrequency = SAI_AUDIO_FREQUENCY_32K;
      break;

    case SAMPLE_FREQ_44100_HZ:
      audioFrequency = SAI_AUDIO_FREQUENCY_44K;
      break;

    case SAMPLE_FREQ_48000_HZ:
      audioFrequency = SAI_AUDIO_FREQUENCY_48K;
      break;

    default:
      Error_Handler();
      return;
  }
  switch(frame_duration)
  {
    case FRAME_DURATION_7_5_MS:
      sample_per_frame = audioFrequency * 75 / 10000;
      break;

    case FRAME_DURATION_10_MS:
      sample_per_frame = audioFrequency * 100 / 10000;
      break;

    default:
      Error_Handler();
      return;
  }

  if (sampling_frequency == SAMPLE_FREQ_44100_HZ)
  {
    /* LC3 need setup like 48Khz for this frequency */
    switch(frame_duration)
    {
      case FRAME_DURATION_7_5_MS:
        sample_per_frame = 360;
        break;

      case FRAME_DURATION_10_MS:
        sample_per_frame = 480;
        break;

      default:
        Error_Handler();
        return;
    }
  }

  __HAL_RCC_I2C3_CLK_ENABLE();

  audio_conf.Device = AUDIO_IN_DEVICE_ANALOG_MIC;
  audio_conf.ChannelsNbr = 1;
  channel_at_src = 1;

  audio_conf.Volume = 50; /* input volume is not used */
  audio_conf.SampleRate = audioFrequency;
  audio_conf.BitsPerSample = AUDIO_RESOLUTION_16B;

  if (BSP_AUDIO_IN_Init(0x00, &audio_conf) != BSP_ERROR_NONE)
  {
    Error_Handler();
  }

  audio_conf.Device = AUDIO_OUT_DEVICE_HEADPHONE;
  audio_conf.ChannelsNbr = 2;
  audio_conf.Volume = Current_Volume;
  if (BSP_AUDIO_OUT_Init(0x00, &audio_conf) != BSP_ERROR_NONE)
  {
    Error_Handler();
  }

  BSP_MIC_Gain_Init(MIC_GAIN_CONFIG);

  /* Start SAI clock without DMA interrupt */
  uint8_t channel_at_snk = 2;   /* SAI is in stereo for Headset */
  uint8_t buffer_nb = 2;        /* double buffer strategy for minimum latency*/
  uint8_t bytes_per_sample = 2;

  Sink_frame_size = sample_per_frame * channel_at_snk * buffer_nb;
  Source_frame_size = sample_per_frame * channel_at_src * buffer_nb;
  /* Start SAI clock without DMA interrupt */
  Init_AudioBuffer((uint8_t *)pSnkBuff,
                   Sink_frame_size * bytes_per_sample,
                   (uint8_t *)pSrcBuff,
                   Source_frame_size * bytes_per_sample);

  /* Release Bus for power consumption optimisation */
  __HAL_RCC_I2C3_CLK_DISABLE();
  MxAudioInit_Flag = 1;

}

void MX_AudioDeInit(void)
{
  if (MxAudioInit_Flag == 1)
  {
    MxAudioInit_Flag = 0;

    __HAL_RCC_I2C3_CLK_ENABLE();

    if (BSP_AUDIO_IN_DeInit(0x00) != BSP_ERROR_NONE)
    {
      Error_Handler();
    }

    if (BSP_AUDIO_OUT_DeInit(0x00) != BSP_ERROR_NONE)
    {
      Error_Handler();
    }

    BSP_I2C3_DeInit();
  }

  AudioClock_Deinit();

#if (CFG_LCD_SUPPORTED == 1)
  Menu_SetNoStreamPage();
#endif /* (CFG_LCD_SUPPORTED == 1) */
}

static void Init_AudioBuffer(uint8_t *pSnkBuff, uint16_t SnkBuffLen, uint8_t *pSrcBuff, uint16_t SrcBuffLen)
{
  /* We start the SAI here but will pause the DMA on the first interrupt.
     The DMA will be relauched synchronized to the BLE transport for mastering audio latency */
  if ((SnkBuffLen  > 0) && (pSnkBuff != NULL))
  {
    if (BSP_AUDIO_OUT_Play(0, (uint8_t *)pSnkBuff, SnkBuffLen) != BSP_ERROR_NONE)
    {
      Error_Handler();
    }
    Play_Req_Pause = 1;
  }
  if ((SrcBuffLen  > 0) && (pSrcBuff != NULL))
  {
    if (BSP_AUDIO_IN_Record(0, (uint8_t *)pSrcBuff, SrcBuffLen) != BSP_ERROR_NONE)
    {
      Error_Handler();
    }
    Record_Req_Pause = 1;
  }
}

void Start_TxAudio(void)
{
  /* Initialize Bus which bas been released for Power consumption optimisation */
  __HAL_RCC_I2C3_CLK_ENABLE();

  APP_NotifyTxAudioCplt(Sink_frame_size);
  if (BSP_AUDIO_OUT_Resume(0) != BSP_ERROR_NONE)
  {
    Error_Handler();
  }

  /* Release Bus for power consumption optimisation */
  __HAL_RCC_I2C3_CLK_DISABLE();

  Audio_Role |= AUDIO_ROLE_SINK;

#if (CFG_LCD_SUPPORTED == 1)
  switch (Codec_Frequency)
  {
    case SAMPLE_FREQ_8000_HZ:
    {
      Menu_SetStreamingPage("8KHz", Audio_Role);
      break;
    }
    case SAMPLE_FREQ_16000_HZ:
    {
      Menu_SetStreamingPage("16KHz", Audio_Role);
      break;
    }
    case SAMPLE_FREQ_24000_HZ:
    {
      Menu_SetStreamingPage("24KHz", Audio_Role);
      break;
    }
    case SAMPLE_FREQ_32000_HZ:
    {
      Menu_SetStreamingPage("32KHz", Audio_Role);
      break;
    }
    case SAMPLE_FREQ_44100_HZ:
    {
      Menu_SetStreamingPage("44.1KHz", Audio_Role);
      break;
    }
    case SAMPLE_FREQ_48000_HZ:
    {
      Menu_SetStreamingPage("48KHz", Audio_Role);
      break;
    }
    default:
    {
      Menu_SetStreamingPage("Unknown Frequency", Audio_Role);
      break;
    }
  }
#endif /* (CFG_LCD_SUPPORTED == 1) */
}

void Stop_TxAudio(void)
{
  /* Initialize Bus which bas been released for Power consumption optimisation */
 __HAL_RCC_I2C3_CLK_ENABLE();

  if (BSP_AUDIO_OUT_Stop(0) != BSP_ERROR_NONE)
  {
    Error_Handler();
  }

  /* Release Bus for power consumption optimisation */
  __HAL_RCC_I2C3_CLK_DISABLE();

  Audio_Role = 0;
}

void BSP_AUDIO_OUT_TransferComplete_CallBack(uint32_t instance)
{
  APP_NotifyTxAudioCplt(Sink_frame_size);
}

void BSP_AUDIO_OUT_HalfTransfer_CallBack(uint32_t instance)
{
  if (MxAudioInit_Flag == 1)
  {
    if (Play_Req_Pause == 1)
    {
      __HAL_RCC_I2C3_CLK_ENABLE();
      /* Pause the DMA that as run one frame only and wait the codec trigger to re run*/
      if (BSP_AUDIO_OUT_Pause(0) != BSP_ERROR_NONE)
      {
        Error_Handler();
      }
      __HAL_RCC_I2C3_CLK_DISABLE();
      Play_Req_Pause = 0;
    }

    APP_NotifyTxAudioHalfCplt();
  }
}

void Start_RxAudio(void)
{
  /* Initialize Bus which bas been released for Power consumption optimisation */
  __HAL_RCC_I2C3_CLK_ENABLE();

  if (BSP_AUDIO_IN_Resume(0) != BSP_ERROR_NONE)
  {
    Error_Handler();
  }

  /* Release Bus for power consumption optimisation */
  __HAL_RCC_I2C3_CLK_DISABLE();

  Audio_Role |= AUDIO_ROLE_SOURCE;

#if (CFG_LCD_SUPPORTED == 1)
  switch (Codec_Frequency)
  {
    case SAMPLE_FREQ_8000_HZ:
    {
      Menu_SetStreamingPage("8KHz", Audio_Role);
      break;
    }
    case SAMPLE_FREQ_16000_HZ:
    {
      Menu_SetStreamingPage("16KHz", Audio_Role);
      break;
    }
    case SAMPLE_FREQ_24000_HZ:
    {
      Menu_SetStreamingPage("24KHz", Audio_Role);
      break;
    }
    case SAMPLE_FREQ_32000_HZ:
    {
      Menu_SetStreamingPage("32KHz", Audio_Role);
      break;
    }
    case SAMPLE_FREQ_44100_HZ:
    {
      Menu_SetStreamingPage("44.1KHz", Audio_Role);
      break;
    }
    case SAMPLE_FREQ_48000_HZ:
    {
      Menu_SetStreamingPage("48KHz", Audio_Role);
      break;
    }
    default:
    {
      Menu_SetStreamingPage("Unknown Frequency", Audio_Role);
      break;
    }
  }
#endif /* (CFG_LCD_SUPPORTED == 1) */
}

void Stop_RxAudio(void)
{
  /* Initialize Bus which bas been released for Power consumption optimisation */
  __HAL_RCC_I2C3_CLK_ENABLE();

  if (BSP_AUDIO_IN_Stop(0) != BSP_ERROR_NONE)
  {
    Error_Handler();
  }

  /* Release Bus for power consumption optimisation */
  __HAL_RCC_I2C3_CLK_DISABLE();
}

void BSP_AUDIO_IN_TransferComplete_CallBack(uint32_t instance)
{
  APP_NotifyRxAudioCplt(Source_frame_size);
}

void BSP_AUDIO_IN_HalfTransfer_CallBack(uint32_t instance)
{
  if (MxAudioInit_Flag == 1)
  {
    if (Record_Req_Pause == 1)
    {
      __HAL_RCC_I2C3_CLK_ENABLE();
      if (BSP_AUDIO_IN_Pause(0) != BSP_ERROR_NONE)
      {
        Error_Handler();
      }
      __HAL_RCC_I2C3_CLK_DISABLE();
      Record_Req_Pause = 0;
    }

    APP_NotifyRxAudioHalfCplt();
  }
}

void Set_Volume(uint8_t Volume)
{
  Current_Volume = VOLUME_VCP_TO_VOLUME_BSP(Volume);

  __HAL_RCC_I2C3_CLK_ENABLE();
  BSP_AUDIO_OUT_SetVolume(0, Current_Volume);
  __HAL_RCC_I2C3_CLK_DISABLE();
}
/* USER CODE END FD_LOCAL_FUNCTIONS */

/*************************************************************
 *
 * WRAP FUNCTIONS
 *
 *************************************************************/

void UTIL_SEQ_Idle( void )
{
#if ( CFG_LPM_LEVEL != 0)
  HAL_SuspendTick();
  UTIL_LPM_EnterLowPower();
  HAL_ResumeTick();
#endif /* CFG_LPM_LEVEL */
  return;
}

void UTIL_SEQ_PreIdle( void )
{
  /* USER CODE BEGIN UTIL_SEQ_PreIdle_1 */

  /* USER CODE END UTIL_SEQ_PreIdle_1 */
#if ( CFG_LPM_LEVEL != 0)
  LL_PWR_ClearFlag_STOP();

  if ( ( system_startup_done != FALSE ) && ( UTIL_LPM_GetMode() == UTIL_LPM_OFFMODE ) )
  {
    APP_SYS_BLE_EnterDeepSleep();
  }

  LL_RCC_ClearResetFlags();

#if defined(STM32WBAXX_SI_CUT1_0)
  /* Wait until HSE is ready */
  while (LL_RCC_HSE_IsReady() == 0);

  UTILS_ENTER_LIMITED_CRITICAL_SECTION(RCC_INTR_PRIO << 4U);
  scm_hserdy_isr();
  UTILS_EXIT_LIMITED_CRITICAL_SECTION();
#endif /* STM32WBAXX_SI_CUT1_0 */
#endif /* CFG_LPM_LEVEL */
  /* USER CODE BEGIN UTIL_SEQ_PreIdle_2 */

  /* USER CODE END UTIL_SEQ_PreIdle_2 */
  return;
}

void UTIL_SEQ_PostIdle( void )
{
  /* USER CODE BEGIN UTIL_SEQ_PostIdle_1 */

  /* USER CODE END UTIL_SEQ_PostIdle_1 */
#if ( CFG_LPM_LEVEL != 0)
  LL_AHB5_GRP1_EnableClock(LL_AHB5_GRP1_PERIPH_RADIO);
  ll_sys_dp_slp_exit();
  UTIL_LPM_SetOffMode(1U << CFG_LPM_LL_DEEPSLEEP, UTIL_LPM_ENABLE);
#endif /* CFG_LPM_LEVEL */
  /* USER CODE BEGIN UTIL_SEQ_PostIdle_2 */
#if (CFG_LPM_STDBY_SUPPORTED == 1)
#if (CFG_JOYSTICK_SUPPORTED == 1)
  if(JOY_StandbyExitFlag == 1){
    /* could reconfigure Joystick here */
    JOY_StandbyExitFlag = 0;
  }
#endif /* CFG_JOYSTICK_SUPPORTED */
#endif /* CFG_LPM_STDBY_SUPPORTED */
  /* USER CODE END UTIL_SEQ_PostIdle_2 */
  return;
}

void BPKACB_Process( void )
{
  UTIL_SEQ_SetTask(1U << CFG_TASK_BPKA, CFG_SEQ_PRIO_0);
}

/**
 * @brief Callback used by Random Number Generator to launch Task to generate Random Numbers
 */
void HWCB_RNG_Process( void )
{
  UTIL_SEQ_SetTask(1U << CFG_TASK_HW_RNG, TASK_PRIO_RNG);
}

void AMM_RegisterBasicMemoryManager (AMM_BasicMemoryManagerFunctions_t * const p_BasicMemoryManagerFunctions)
{
  /* Fulfill the function handle */
  p_BasicMemoryManagerFunctions->Init = AMM_WrapperInit;
  p_BasicMemoryManagerFunctions->Allocate = AMM_WrapperAllocate;
  p_BasicMemoryManagerFunctions->Free = AMM_WrapperFree;
}

void AMM_ProcessRequest(void)
{
  /* Trigger to call Advance Memory Manager process function */
  UTIL_SEQ_SetTask(1U << CFG_TASK_AMM, CFG_SEQ_PRIO_0);
}

void FM_ProcessRequest(void)
{
  /* Trigger to call Flash Manager process function */
  UTIL_SEQ_SetTask(1U << CFG_TASK_FLASH_MANAGER, CFG_SEQ_PRIO_0);
}

#if ((CFG_LOG_SUPPORTED == 0) && (CFG_LPM_LEVEL != 0))
/* RNG module turn off HSI clock when traces are not used and low power used */
void RNG_KERNEL_CLK_OFF(void)
{
  /* USER CODE BEGIN RNG_KERNEL_CLK_OFF_1 */

  /* USER CODE END RNG_KERNEL_CLK_OFF_1 */
  LL_RCC_HSI_Disable();
  /* USER CODE BEGIN RNG_KERNEL_CLK_OFF_2 */

  /* USER CODE END RNG_KERNEL_CLK_OFF_2 */
}

/* SCM module turn off HSI clock when traces are not used and low power used */
void SCM_HSI_CLK_OFF(void)
{
  /* USER CODE BEGIN SCM_HSI_CLK_OFF_1 */

  /* USER CODE END SCM_HSI_CLK_OFF_1 */
  LL_RCC_HSI_Disable();
  /* USER CODE BEGIN SCM_HSI_CLK_OFF_2 */

  /* USER CODE END SCM_HSI_CLK_OFF_2 */
}
#endif /* ((CFG_LOG_SUPPORTED == 0) && (CFG_LPM_LEVEL != 0)) */

#if (CFG_LOG_SUPPORTED != 0)
void UTIL_ADV_TRACE_PreSendHook(void)
{
#if (CFG_LPM_LEVEL != 0)
  /* Disable Stop mode before sending a LOG message over UART */
  UTIL_LPM_SetStopMode(1U << CFG_LPM_LOG, UTIL_LPM_DISABLE);
#endif /* (CFG_LPM_LEVEL != 0) */
  /* USER CODE BEGIN UTIL_ADV_TRACE_PreSendHook */

  /* USER CODE END UTIL_ADV_TRACE_PreSendHook */
}

void UTIL_ADV_TRACE_PostSendHook(void)
{
#if (CFG_LPM_LEVEL != 0)
  /* Enable Stop mode after LOG message over UART sent */
  UTIL_LPM_SetStopMode(1U << CFG_LPM_LOG, UTIL_LPM_ENABLE);
#endif /* (CFG_LPM_LEVEL != 0) */
  /* USER CODE BEGIN UTIL_ADV_TRACE_PostSendHook */

  /* USER CODE END UTIL_ADV_TRACE_PostSendHook */
}

#endif /* (CFG_LOG_SUPPORTED != 0) */

/* USER CODE BEGIN FD_WRAP_FUNCTIONS */
void HAL_GPIO_EXTI_Falling_Callback(uint16_t GPIO_Pin)
{
  HAL_GPIO_EXTI_Rising_Callback(GPIO_Pin);
}

/* USER CODE END FD_WRAP_FUNCTIONS */
